import re
import json
import logging
from contextlib import AsyncExitStack
from typing import ClassVar, Pattern

import aiohttp

logger = logging.getLogger(__name__)


class FrigateConfig:
    _LOCALHOST_REGEX: ClassVar[Pattern[str]] = re.compile("127\\.0\\.0\\.1|localhost")
    _URL_ADDRESS_REGEX: ClassVar[Pattern[str]] = re.compile("(^|(?<=://)|(?<=@))[a-z0-9.\\-]+(:[0-9]+)?($|(?=/))")
    _WYZE_CAMERAS: ClassVar[dict[str, str]] = {"back_yard_cam": "192.168.0.202:554"}

    def __init__(self, frigate_base_url: str = "http://frigate:5000") -> None:
        self._frigate_config_url = f"{frigate_base_url}/api/config"
        self._config = {}

    async def refresh(self) -> None:
        logger.debug("Fetching Frigate configuration")
        try:
            async with AsyncExitStack() as exit_stack:
                aiohttp_session = await exit_stack.enter_async_context(aiohttp.ClientSession(raise_for_status=True))
                response = await exit_stack.enter_async_context(aiohttp_session.get(self._frigate_config_url))
                self._config = json.loads(await response.read())
        except Exception as e:
            if self._config:
                logger.error("Failed to fetch Frigate config, falling back to previous value", exc_info=e)
            else:
                raise
        logger.debug("Fetched Frigate configuration")

    @property
    def active_cameras(self) -> list[str]:
        if "cameras" not in self._config:
            raise ValueError("Configuration not yet fetched from Frigate")

        return [camera for camera in self._config["cameras"] if self._config["cameras"][camera].get("enabled", True)]

    @property
    def active_camera_urls(self) -> dict[str, str]:
        return {camera: self._replace_localhost(self._config["cameras"][camera]["ffmpeg"]["inputs"][0]["path"]) for camera in self.active_cameras}

    @property
    def active_camera_addresses(self) -> dict[str, str]:
        active_cameras = self.active_cameras
        return {camera: self._get_address_from_url(url) for camera, url in self.active_camera_urls.items()}

    @classmethod
    def _replace_localhost(cls, url: str) -> str:
        return cls._LOCALHOST_REGEX.sub("frigate", url, 1)

    @classmethod
    def _get_address_from_url(cls, url: str) -> str:
        match = cls._URL_ADDRESS_REGEX.search(url.lower())
        if match is None:
            raise ValueError(f"Failed to retrieve address from {url=}")

        # Handle case of wyze-bridge and hardcode cameras
        if match.group().startswith("wyze-bridge"):
            wyze_camera = url.lower().rsplit("/", 1)[-1].replace("-", "_")
            return cls._WYZE_CAMERAS[wyze_camera]
        return match.group()
